
import enums.TransactionType
import model.NeftTransaction
import model.Teller
import model.TransactionRequest
import model.TransferTransaction
import java.time.format.DateTimeFormatter

class TransactionFactory(private val transactionRequest: TransactionRequest, private val teller: Teller) {

    private val date = transactionRequest.date.format(DateTimeFormatter.ofPattern("dd-MM-yyyy"))
    private val rrn = transactionRequest.date.format(DateTimeFormatter.ofPattern("ddMM")) + transactionRequest.transactionNumber.takeLast(4)

    private fun createTransferTransaction(): TransferTransaction {
        return TransferTransaction(
            bankCode = transactionRequest.dccbCode.padStart(4, '0'),
            branchCode = transactionRequest.branchCode.padStart(3,'0'),
            cbsTellerId = teller.tellerId,
            cbsTellerUserIdType = teller.userType,
            queIdType = "5",
            description = "${TransactionType.TRANSFER.code}For Checking",
            priority = "1",
            cbsTellerCapability = teller.capability,
            txnScreenNo = TransactionType.TRANSFER.code.padStart(6, '0'),
            txnAmt = transactionRequest.amount,
            txnDate = date,
            sourceAcctNo = transactionRequest.pacsCurrentAccountNumber,
            destinationAcctNo = transactionRequest.linkedCBSAccountNumber,
            narration = "TRF to member A/C for NEFT RTGS",
            sourceTxnNo = "1045",
            sourceStat = "A/P",
            apiType = "OUTWARD_QUEUE_POSTING",
            remitterName = transactionRequest.remitterName,
            rrn = rrn + "1"
        )

    }

    private fun createNEFTTransaction(): NeftTransaction {
        return NeftTransaction(
            bankCode = transactionRequest.dccbCode,
            branchCode = transactionRequest.branchCode.padStart(3,'0'),
            cbsTellerId = teller.tellerId,
            cbsTellerUserIdType = teller.userType,
            queIdType = "5",
            description = "${TransactionType.NEFT.code}For Checking",
            priority = "1",
            cbsTellerCapability = teller.capability,
            txnScreenNo = TransactionType.NEFT.code.padStart(6, '0'),
            txnAmt = transactionRequest.amount,
            txnDate = date,
            sourceAcctNo = transactionRequest.linkedCBSAccountNumber,
            destinationAcctNo = transactionRequest.neftBeneficiaryAccountNumber,
            narration = "TRF to member A/C for NEFT RTGS",
            sourceTxnNo = TransactionType.NEFT.code,
            sourceStat = "A/P",
            apiType = "OUTWARD_QUEUE_POSTING",
            remitterName = transactionRequest.remitterName,
            ifscCode = transactionRequest.ifscCode,
            rrn = rrn + "2"
        )
    }

    fun createTransactionPair(): Pair<TransferTransaction, NeftTransaction> {
        return Pair(createTransferTransaction(), createNEFTTransaction())
    }
}